/*
     This file is part of GNUnet

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * sessionkey - standard communication with another node.
 * encrypt/decrypt - for the content using the hashcode
 * @author Christian Grothoff
 * @author Ioana Patrascu
 * @file include/util/symcipher.h
 **/

#ifndef SYMCIPHER_H
#define SYMCIPHER_H

#include "config.h"
#include <stdlib.h>
#include <string.h>

#include "util/configuration.h"
#include "util/hostkey.h"
#include "util/logging.h"
#include "util/returnvalues.h"
#include "util/storage.h"

/** length of the sessionkey in bytes
   (128 BIT sessionkey) */
#define SESSIONKEY_LEN (128/8)

/** size of blowfish key in bytes */
#define BF_KEYSIZE 16

/** this unsigned short is 64 bit, blowfish */
#define BLOWFISH_BLOCK_LENGTH 8 

/** value for the IV in the streamcipher for the
    link-to-link encryption **/
#define INITVALUE "GNUnet!!"

/** 
 * type for session keys 
 **/
typedef struct {
  unsigned char key[SESSIONKEY_LEN];
} SESSIONKEY;

/**
 * Create a new Session key.
 **/
void makeSessionkey(SESSIONKEY * key);

/**
 * Encrypt a block with the public key of another
 * host that uses the same cyper.
 * @param block the block to encrypt
 * @param len the size of the block
 * @param sessionkey the key used to encrypt
 * @param iv the initialization vector to use, use INITVALUE
 *        for streams.
 * @returns the size of the encrypted block, -1 for errors
 **/
int encryptBlock(void * block, 
		 unsigned short len,
		 SESSIONKEY * sessionkey,
		 unsigned char * iv,
		 void * result);

/**
 * Decrypt a given block with the sessionkey.
 * @param sessionkey the key used to decrypt
 * @param block the data to decrypt, encoded as returned by encrypt
 * @param size how big is the block?
 * @param iv the initialization vector to use, use INITVALUE
 *        for streams.
 * @param result address to store the result at
 * @return -1 on failure, size of decrypted block on success
 **/
int decryptBlock(SESSIONKEY * sessionkey, 
		 void * block,
		 unsigned short size,
		 unsigned char * iv,
		 void * result);

/* end of symcipher.h */
#endif
